@@ -620,6 +620,12 @@ POSSIBILITY OF SUCH DAMAGE.
 #define __has_builtin(x) 0  // for non-clang compilers
 #endif
 
+#ifdef __cpp_guaranteed_copy_elision
+#define TORRENT_RVO(x) x
+#else
+#define TORRENT_RVO(x) std::move(x)
+#endif
+
 #if (TORRENT_HAS_SSE && defined __GNUC__)
 #	define TORRENT_HAS_BUILTIN_CLZ 1
 #elif (TORRENT_HAS_ARM && defined __GNUC__ && !defined __clang__)
