@@ -1,6 +1,8 @@
 /*
 
-Copyright (c) 2007-2018, Arvid Norberg
+Copyright (c) 2007, 2013, 2017, 2019, Arvid Norberg
+Copyright (c) 2015, Steven Siloti
+Copyright (c) 2016, Alden Torres
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -43,6 +45,7 @@ namespace libtorrent {
 
 	struct torrent_plugin;
 	struct torrent_handle;
+	struct client_data_t;
 
 	// constructor function for the smart ban extension. The extension keeps
 	// track of the data peers have sent us for failing pieces and once the
@@ -50,7 +53,7 @@ namespace libtorrent {
 	// out to have sent corrupt data.
 	// This function can either be passed in the add_torrent_params::extensions
 	// field, or via torrent_handle::add_extension().
-	TORRENT_EXPORT std::shared_ptr<torrent_plugin> create_smart_ban_plugin(torrent_handle const&, void*);
+	TORRENT_EXPORT std::shared_ptr<torrent_plugin> create_smart_ban_plugin(torrent_handle const&, client_data_t);
 }
 
 #endif // TORRENT_DISABLE_EXTENSIONS
