@@ -1,6 +1,8 @@
 /*
 
-Copyright (c) 2012-2018, Arvid Norberg
+Copyright (c) 2014-2017, 2019, 2021, Arvid Norberg
+Copyright (c) 2016-2017, Alden Torres
+Copyright (c) 2018, Steven Siloti
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -37,6 +39,7 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/address.hpp"
 #include "libtorrent/socket.hpp"
 #include "libtorrent/peer_info.hpp" // for peer_source_flags_t
+#include "libtorrent/info_hash.hpp"
 #include "libtorrent/aux_/string_ptr.hpp"
 #include "libtorrent/string_view.hpp"
 
@@ -56,7 +59,7 @@ namespace libtorrent {
 		torrent_peer(std::uint16_t port, bool connectable, peer_source_flags_t src);
 #if TORRENT_USE_ASSERTS
 		torrent_peer(torrent_peer const&) = default;
-		torrent_peer& operator=(torrent_peer const&) = default;
+		torrent_peer& operator=(torrent_peer const&) & = default;
 		~torrent_peer() { TORRENT_ASSERT(in_use); in_use = false; }
 #endif
 
@@ -74,6 +77,8 @@ namespace libtorrent {
 		std::string to_string() const;
 #endif
 
+		protocol_version protocol() { return protocol_v2 ? protocol_version::V2 : protocol_version::V1; }
+
 		// this is the accumulated amount of
 		// uploaded and downloaded data to this
 		// torrent_peer. It only accounts for what was
@@ -207,6 +212,8 @@ namespace libtorrent {
 		// so, any torrent_peer with the web_seed bit set, is
 		// never considered a connect candidate
 		bool web_seed:1;
+		// this peer supports protocol version 2
+		bool protocol_v2:1;
 #if TORRENT_USE_ASSERTS
 		bool in_use = true;
 #endif
@@ -214,9 +221,9 @@ namespace libtorrent {
 
 	struct TORRENT_EXTRA_EXPORT ipv4_peer : torrent_peer
 	{
-		ipv4_peer(tcp::endpoint const& ip, bool connectable, peer_source_flags_t src);
+		ipv4_peer(tcp::endpoint const& ep, bool connectable, peer_source_flags_t src);
 		ipv4_peer(ipv4_peer const& p);
-		ipv4_peer& operator=(ipv4_peer const& p);
+		ipv4_peer& operator=(ipv4_peer const& p) &;
 
 		address_v4 addr;
 	};
@@ -224,11 +231,11 @@ namespace libtorrent {
 #if TORRENT_USE_I2P
 	struct TORRENT_EXTRA_EXPORT i2p_peer : torrent_peer
 	{
-		i2p_peer(string_view dst, bool connectable, peer_source_flags_t src);
+		i2p_peer(string_view dest, bool connectable, peer_source_flags_t src);
 		i2p_peer(i2p_peer const&) = delete;
 		i2p_peer& operator=(i2p_peer const&) = delete;
 		i2p_peer(i2p_peer&&) = default;
-		i2p_peer& operator=(i2p_peer&&) = default;
+		i2p_peer& operator=(i2p_peer&&) & = default;
 
 		aux::string_ptr destination;
 	};
@@ -236,45 +243,70 @@ namespace libtorrent {
 
 	struct TORRENT_EXTRA_EXPORT ipv6_peer : torrent_peer
 	{
-		ipv6_peer(tcp::endpoint const& ip, bool connectable, peer_source_flags_t src);
+		ipv6_peer(tcp::endpoint const& ep, bool connectable, peer_source_flags_t src);
 		ipv6_peer(ipv6_peer const& p);
 
 		const address_v6::bytes_type addr;
 	};
 
+	// if we have i2p peer addresses, sort them *after* all IP addresses
 	struct peer_address_compare
 	{
 		bool operator()(torrent_peer const* lhs, address const& rhs) const
 		{
+#if TORRENT_USE_I2P
+			if (lhs->is_i2p_addr) return false;
+#endif
 			return lhs->address() < rhs;
 		}
 
 		bool operator()(address const& lhs, torrent_peer const* rhs) const
 		{
+#if TORRENT_USE_I2P
+			if (rhs->is_i2p_addr) return true;
+#endif
 			return lhs < rhs->address();
 		}
 
 #if TORRENT_USE_I2P
 		bool operator()(torrent_peer const* lhs, string_view rhs) const
 		{
-			return lhs->dest().compare(rhs) < 0;
+			if (!lhs->is_i2p_addr) return true;
+			return lhs->dest() < rhs;
 		}
 
 		bool operator()(string_view lhs, torrent_peer const* rhs) const
 		{
-			return lhs.compare(rhs->dest()) < 0;
+			if (!rhs->is_i2p_addr) return false;
+			return lhs < rhs->dest();
 		}
 #endif
 
 		bool operator()(torrent_peer const* lhs, torrent_peer const* rhs) const
 		{
 #if TORRENT_USE_I2P
-			if (rhs->is_i2p_addr == lhs->is_i2p_addr)
-				return lhs->dest().compare(rhs->dest()) < 0;
+			if (lhs->is_i2p_addr != rhs->is_i2p_addr)
+				return lhs->is_i2p_addr < rhs->is_i2p_addr;
+
+			if (lhs->is_i2p_addr)
+				return lhs->dest() < rhs->dest();
 #endif
 			return lhs->address() < rhs->address();
 		}
 	};
+
+	inline bool torrent_peer_equal(torrent_peer const* lhs, torrent_peer const* rhs)
+	{
+#if TORRENT_USE_I2P
+		if (lhs->is_i2p_addr != rhs->is_i2p_addr)
+			return false;
+
+		if (lhs->is_i2p_addr)
+			return lhs->dest() == rhs->dest();
+#endif
+		return lhs->address() == rhs->address()
+			&& lhs->port == rhs->port;
+	}
 }
 
 #endif
